using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

namespace Hopfield3
{
	public class FrmMain : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;

		// network
		private HNet hnet = null;
		private int pat = 0; // selected pattern
		private string fileName = "";

		// settings
		public bool showWeights = true;
		
		// consts
		public const int gridSpace = 8;
		public const int maxStates = 8;

		// graphics
		private System.Drawing.SolidBrush[] brush = new System.Drawing.SolidBrush[maxStates];
		private System.Drawing.SolidBrush brushSel;
		private System.Drawing.SolidBrush brushSel0;
		private System.Drawing.SolidBrush brushSel1;
		private System.Drawing.Graphics gNet;
		private System.Drawing.Graphics gSet;
		int simulated = -1;
		int animDelay = 100;

		// layout
		int widthHalf;
		int height;
		int gridSize;
		int width;
		int circleSize;
		float diameter;
		int cx;
		int cy;

		// interaction
		int lx;
		int ly;
		bool action = false;
		bool mouse = false;

		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.TabControl tc;
		private System.Windows.Forms.TabPage tpNet;
		private System.Windows.Forms.TabPage tpSet;
		private System.Windows.Forms.ListBox lb;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem mnuWeights;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.MenuItem mnuOpen;
		private System.Windows.Forms.MenuItem mnuSave;
		private System.Windows.Forms.MenuItem mnuAdd;
		private System.Windows.Forms.MenuItem mnuRemove;
		private System.Windows.Forms.MenuItem mnuSimulate;
		private System.Windows.Forms.MenuItem mnuLearn;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem mnuQuit;
		private System.Windows.Forms.ToolBar tb;
		private System.Windows.Forms.ImageList il;
		private System.Windows.Forms.MenuItem mnuNew;
		private System.Windows.Forms.SaveFileDialog sfd;
		private System.Windows.Forms.OpenFileDialog ofd;
		private System.Windows.Forms.MenuItem menuItem16;
		private System.Windows.Forms.MenuItem menuItem17;
		private System.Windows.Forms.MenuItem mnuBi;
		private System.Windows.Forms.MenuItem mnuTri;
		private System.Windows.Forms.MenuItem mnuNetwork;
		private System.Windows.Forms.MenuItem mnuPatterns;
		private System.Windows.Forms.MenuItem mnu1000;
		private System.Windows.Forms.MenuItem mnu100;
		private System.Windows.Forms.MenuItem mnu10;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem mnuTest;
		private System.Windows.Forms.MenuItem mnuAbout;
		private System.Windows.Forms.MainMenu mm;
		private System.Windows.Forms.ToolBarButton tbNew;
		private System.Windows.Forms.ToolBarButton tbOpen;
		private System.Windows.Forms.ToolBarButton tbSave;
		private System.Windows.Forms.ToolBarButton toolBarButton1;
		private System.Windows.Forms.ToolBarButton toolBarButton2;
		private System.Windows.Forms.ToolBarButton tbQuit;
		private System.Windows.Forms.ToolBarButton tbAbout;
		private System.Windows.Forms.ToolBarButton tbLearn;
		private System.Windows.Forms.ToolBarButton tbSimulate;
		private System.Windows.Forms.ToolBarButton tbStop;
		private System.Windows.Forms.ToolBarButton tbAdd;
		private System.Windows.Forms.ToolBarButton tbRemove;
		private System.Windows.Forms.ToolBarButton toolBarButton3;
		private System.Windows.Forms.ToolBarButton tbWeights;
		private System.Windows.Forms.ToolBarButton tbStates;
		private System.Windows.Forms.MenuItem mnuStop;
		private System.Windows.Forms.MenuItem mnuRandom;
		private System.Windows.Forms.MenuItem mnuAll;
		private System.Windows.Forms.StatusBar sb;

		public FrmMain()
		{
			InitializeComponent();
			
			brushSel0 = new System.Drawing.SolidBrush(System.Drawing.Color.FromArgb(255,127,63)); 
			brushSel1 = new System.Drawing.SolidBrush(System.Drawing.Color.FromArgb(255,63,127)); 

			InitBrushes();

			tc_Click(null, null);
		}

		private void InitBrushes()
		{
			if (hnet == null)
			{
				for (int i = 0; i < maxStates; i++)
					brush[i] = new System.Drawing.SolidBrush(System.Drawing.Color.FromArgb(0,0,0)); 
			}
			else
			{
				int d = 255 / (hnet.States - 1);
				for (int i = 0; i < hnet.States; i++)
					brush[i] = new System.Drawing.SolidBrush(System.Drawing.Color.FromArgb(i*d,i*d,i*d)); 
			}
		}

		public void StatusRefresh()
		{
			if (hnet == null)
			{
				sb.Text = "[No network loaded]";
			}
			else
			{
				sb.Text = "[" + fileName + "] ";
				if (hnet.States == 2)
					sb.Text += "bi-polar ";
				else if (hnet.States == 3)
					sb.Text += "tri-polar ";
				else if (hnet.States == 4)
					sb.Text += "tetra-polar ";
				else
					sb.Text += hnet.States.ToString() + "-polar ";
				//sb.Text += (hnet.States == 2) ? "bi-polar " : "tri-polar ";
				sb.Text += "network " +
					hnet.DimensionX.ToString() + "x" +
					hnet.DimensionY.ToString() + " neurons, ";
				sb.Text += hnet.Patterns.ToString() + " patterns ";
				if (hnet.Recognized > -1)
					sb.Text += "(recognized pattern #" + hnet.Recognized.ToString() + ")";
			}
		}
		
		public void AllRefresh()
		{
			MenuRefresh();
			FillPatterns();
			StatusRefresh();
			Refresh();
			lx = ly = -1;
		}
		
		public void MenuRefresh()
		{
			mnuWeights.Checked = showWeights; 
			tbWeights.Pushed = showWeights;
			mnu10.Checked   = (animDelay == 10);
			mnu100.Checked  = (animDelay == 100);
			mnu1000.Checked = (animDelay == 1000);

			if (hnet == null)
			{
				mnuBi.Checked = false;
				mnuTri.Checked = false;
				mnuSave.Enabled = false;
				tbSave.Enabled = false;
				tbLearn.Enabled = false;
				tbSimulate.Enabled = false;
				tbStop.Enabled = false;
				tbAdd.Enabled = false;
				tbRemove.Enabled = false;
				tbStates.Enabled = false;
				mnuStop.Enabled = false;
				mnuNetwork.Enabled = false;
				mnuPatterns.Enabled = false;
				mnuTest.Enabled = false;
			}
			else
			{
				mnuBi.Checked = (hnet.States == 2);
				mnuTri.Checked = (hnet.States == 3);
				mnuSave.Enabled = true;
				tbSave.Enabled = true;
				tbLearn.Enabled = !action;
				tbSimulate.Enabled = !action;
				tbStop.Enabled = action;
				tbAdd.Enabled = !action;
				tbRemove.Enabled = !action;
				tbStates.Enabled = !action;
				mnuStop.Enabled = action;
				mnuNetwork.Enabled = true;
				mnuPatterns.Enabled = true;
				mnuTest.Enabled = true;
			}
		}

		public void NetworkNew(int x, int y, int states)
		{
			hnet = new HNet(x,y,states); 
			InitBrushes();
			fileName = "net1";
			showWeights = (hnet.DimensionX * hnet.DimensionY <= 50);
			tc_Click(null, null);
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FrmMain));
			this.tc = new System.Windows.Forms.TabControl();
			this.tpNet = new System.Windows.Forms.TabPage();
			this.tpSet = new System.Windows.Forms.TabPage();
			this.lb = new System.Windows.Forms.ListBox();
			this.mm = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.mnuNew = new System.Windows.Forms.MenuItem();
			this.mnuOpen = new System.Windows.Forms.MenuItem();
			this.mnuSave = new System.Windows.Forms.MenuItem();
			this.menuItem11 = new System.Windows.Forms.MenuItem();
			this.mnuQuit = new System.Windows.Forms.MenuItem();
			this.mnuPatterns = new System.Windows.Forms.MenuItem();
			this.mnuAdd = new System.Windows.Forms.MenuItem();
			this.mnuRemove = new System.Windows.Forms.MenuItem();
			this.mnuNetwork = new System.Windows.Forms.MenuItem();
			this.mnuLearn = new System.Windows.Forms.MenuItem();
			this.mnuSimulate = new System.Windows.Forms.MenuItem();
			this.mnuStop = new System.Windows.Forms.MenuItem();
			this.menuItem17 = new System.Windows.Forms.MenuItem();
			this.menuItem16 = new System.Windows.Forms.MenuItem();
			this.mnuBi = new System.Windows.Forms.MenuItem();
			this.mnuTri = new System.Windows.Forms.MenuItem();
			this.mnuTest = new System.Windows.Forms.MenuItem();
			this.mnuRandom = new System.Windows.Forms.MenuItem();
			this.mnuAll = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.mnuWeights = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.mnu10 = new System.Windows.Forms.MenuItem();
			this.mnu100 = new System.Windows.Forms.MenuItem();
			this.mnu1000 = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.mnuAbout = new System.Windows.Forms.MenuItem();
			this.sb = new System.Windows.Forms.StatusBar();
			this.tb = new System.Windows.Forms.ToolBar();
			this.tbNew = new System.Windows.Forms.ToolBarButton();
			this.tbOpen = new System.Windows.Forms.ToolBarButton();
			this.tbSave = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton1 = new System.Windows.Forms.ToolBarButton();
			this.tbAdd = new System.Windows.Forms.ToolBarButton();
			this.tbRemove = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton3 = new System.Windows.Forms.ToolBarButton();
			this.tbStates = new System.Windows.Forms.ToolBarButton();
			this.tbLearn = new System.Windows.Forms.ToolBarButton();
			this.tbSimulate = new System.Windows.Forms.ToolBarButton();
			this.tbStop = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton2 = new System.Windows.Forms.ToolBarButton();
			this.tbWeights = new System.Windows.Forms.ToolBarButton();
			this.tbAbout = new System.Windows.Forms.ToolBarButton();
			this.tbQuit = new System.Windows.Forms.ToolBarButton();
			this.il = new System.Windows.Forms.ImageList(this.components);
			this.sfd = new System.Windows.Forms.SaveFileDialog();
			this.ofd = new System.Windows.Forms.OpenFileDialog();
			this.tc.SuspendLayout();
			this.tpSet.SuspendLayout();
			this.SuspendLayout();
			// 
			// tc
			// 
			this.tc.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.tc.Controls.AddRange(new System.Windows.Forms.Control[] {
																			 this.tpNet,
																			 this.tpSet});
			this.tc.Location = new System.Drawing.Point(0, 32);
			this.tc.Name = "tc";
			this.tc.SelectedIndex = 0;
			this.tc.Size = new System.Drawing.Size(496, 250);
			this.tc.TabIndex = 0;
			this.tc.Click += new System.EventHandler(this.tc_Click);
			this.tc.SizeChanged += new System.EventHandler(this.tc_SizeChanged);
			// 
			// tpNet
			// 
			this.tpNet.BackColor = System.Drawing.Color.LightSteelBlue;
			this.tpNet.Location = new System.Drawing.Point(4, 22);
			this.tpNet.Name = "tpNet";
			this.tpNet.Size = new System.Drawing.Size(384, 184);
			this.tpNet.TabIndex = 0;
			this.tpNet.Text = "Network status";
			this.tpNet.MouseUp += new System.Windows.Forms.MouseEventHandler(this.tpNet_MouseUp);
			this.tpNet.Paint += new System.Windows.Forms.PaintEventHandler(this.tpNet_Paint);
			this.tpNet.MouseMove += new System.Windows.Forms.MouseEventHandler(this.tpNet_MouseMove);
			this.tpNet.MouseDown += new System.Windows.Forms.MouseEventHandler(this.tpNet_MouseDown);
			// 
			// tpSet
			// 
			this.tpSet.BackColor = System.Drawing.Color.DarkSeaGreen;
			this.tpSet.Controls.AddRange(new System.Windows.Forms.Control[] {
																				this.lb});
			this.tpSet.Location = new System.Drawing.Point(4, 22);
			this.tpSet.Name = "tpSet";
			this.tpSet.Size = new System.Drawing.Size(488, 224);
			this.tpSet.TabIndex = 1;
			this.tpSet.Text = "Patterns";
			this.tpSet.MouseUp += new System.Windows.Forms.MouseEventHandler(this.tpNet_MouseUp);
			this.tpSet.Paint += new System.Windows.Forms.PaintEventHandler(this.tpSet_Paint);
			this.tpSet.MouseMove += new System.Windows.Forms.MouseEventHandler(this.tpSet_MouseMove);
			this.tpSet.MouseDown += new System.Windows.Forms.MouseEventHandler(this.tpSet_MouseDown);
			// 
			// lb
			// 
			this.lb.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.lb.BackColor = System.Drawing.Color.DarkOliveGreen;
			this.lb.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.lb.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lb.ForeColor = System.Drawing.Color.Gold;
			this.lb.IntegralHeight = false;
			this.lb.Location = new System.Drawing.Point(176, 6);
			this.lb.Name = "lb";
			this.lb.Size = new System.Drawing.Size(304, 212);
			this.lb.TabIndex = 0;
			this.lb.SelectedIndexChanged += new System.EventHandler(this.lb_SelectedIndexChanged);
			// 
			// mm
			// 
			this.mm.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																			   this.menuItem1,
																			   this.mnuPatterns,
																			   this.mnuNetwork,
																			   this.mnuTest,
																			   this.menuItem3,
																			   this.menuItem4});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuNew,
																					  this.mnuOpen,
																					  this.mnuSave,
																					  this.menuItem11,
																					  this.mnuQuit});
			this.menuItem1.Text = "File";
			// 
			// mnuNew
			// 
			this.mnuNew.Index = 0;
			this.mnuNew.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.mnuNew.Text = "New...";
			this.mnuNew.Click += new System.EventHandler(this.mnuNew_Click);
			// 
			// mnuOpen
			// 
			this.mnuOpen.Index = 1;
			this.mnuOpen.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.mnuOpen.Text = "Open...";
			this.mnuOpen.Click += new System.EventHandler(this.mnuOpen_Click);
			// 
			// mnuSave
			// 
			this.mnuSave.Index = 2;
			this.mnuSave.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.mnuSave.Text = "Save...";
			this.mnuSave.Click += new System.EventHandler(this.mnuSave_Click);
			// 
			// menuItem11
			// 
			this.menuItem11.Index = 3;
			this.menuItem11.Text = "-";
			// 
			// mnuQuit
			// 
			this.mnuQuit.Index = 4;
			this.mnuQuit.Shortcut = System.Windows.Forms.Shortcut.CtrlQ;
			this.mnuQuit.Text = "Quit";
			this.mnuQuit.Click += new System.EventHandler(this.mnuQuit_Click);
			// 
			// mnuPatterns
			// 
			this.mnuPatterns.Index = 1;
			this.mnuPatterns.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						this.mnuAdd,
																						this.mnuRemove});
			this.mnuPatterns.Text = "Patterns";
			// 
			// mnuAdd
			// 
			this.mnuAdd.Index = 0;
			this.mnuAdd.Shortcut = System.Windows.Forms.Shortcut.Ins;
			this.mnuAdd.Text = "Add";
			this.mnuAdd.Click += new System.EventHandler(this.mnuAdd_Click);
			// 
			// mnuRemove
			// 
			this.mnuRemove.Index = 1;
			this.mnuRemove.Shortcut = System.Windows.Forms.Shortcut.Del;
			this.mnuRemove.Text = "Remove";
			this.mnuRemove.Click += new System.EventHandler(this.mnuRemove_Click);
			// 
			// mnuNetwork
			// 
			this.mnuNetwork.Index = 2;
			this.mnuNetwork.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					   this.mnuLearn,
																					   this.mnuSimulate,
																					   this.mnuStop,
																					   this.menuItem17,
																					   this.menuItem16});
			this.mnuNetwork.Text = "Network";
			// 
			// mnuLearn
			// 
			this.mnuLearn.Index = 0;
			this.mnuLearn.Shortcut = System.Windows.Forms.Shortcut.F5;
			this.mnuLearn.Text = "Learn patterns";
			this.mnuLearn.Click += new System.EventHandler(this.mnuLearn_Click);
			// 
			// mnuSimulate
			// 
			this.mnuSimulate.Index = 1;
			this.mnuSimulate.Shortcut = System.Windows.Forms.Shortcut.F6;
			this.mnuSimulate.Text = "Simulate";
			this.mnuSimulate.Click += new System.EventHandler(this.mnuSimulate_Click);
			// 
			// mnuStop
			// 
			this.mnuStop.Index = 2;
			this.mnuStop.Shortcut = System.Windows.Forms.Shortcut.F7;
			this.mnuStop.Text = "Stop";
			this.mnuStop.Click += new System.EventHandler(this.mnuStop_Click);
			// 
			// menuItem17
			// 
			this.menuItem17.Index = 3;
			this.menuItem17.Text = "-";
			// 
			// menuItem16
			// 
			this.menuItem16.Index = 4;
			this.menuItem16.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					   this.mnuBi,
																					   this.mnuTri});
			this.menuItem16.Text = "States";
			// 
			// mnuBi
			// 
			this.mnuBi.Index = 0;
			this.mnuBi.Text = "2 (bi-polar)";
			this.mnuBi.Click += new System.EventHandler(this.mnuBi_Click);
			// 
			// mnuTri
			// 
			this.mnuTri.Index = 1;
			this.mnuTri.Text = "3 (tri-polar)";
			this.mnuTri.Click += new System.EventHandler(this.mnuTri_Click);
			// 
			// mnuTest
			// 
			this.mnuTest.Index = 3;
			this.mnuTest.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.mnuRandom,
																					this.mnuAll});
			this.mnuTest.Text = "Test";
			// 
			// mnuRandom
			// 
			this.mnuRandom.Index = 0;
			this.mnuRandom.Text = "Random...";
			this.mnuRandom.Click += new System.EventHandler(this.mnuRandom_Click);
			// 
			// mnuAll
			// 
			this.mnuAll.Index = 1;
			this.mnuAll.Text = "All...";
			this.mnuAll.Click += new System.EventHandler(this.mnuWhole_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 4;
			this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuWeights,
																					  this.menuItem2});
			this.menuItem3.Text = "View settings";
			// 
			// mnuWeights
			// 
			this.mnuWeights.Index = 0;
			this.mnuWeights.Text = "Showing weights";
			this.mnuWeights.Click += new System.EventHandler(this.mnuWeights_Click);
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 1;
			this.menuItem2.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnu10,
																					  this.mnu100,
																					  this.mnu1000});
			this.menuItem2.Text = "Animation delay";
			// 
			// mnu10
			// 
			this.mnu10.Index = 0;
			this.mnu10.Text = "10 ms";
			this.mnu10.Click += new System.EventHandler(this.mnu10_Click);
			// 
			// mnu100
			// 
			this.mnu100.Index = 1;
			this.mnu100.Text = "100 ms";
			this.mnu100.Click += new System.EventHandler(this.mnu100_Click);
			// 
			// mnu1000
			// 
			this.mnu1000.Index = 2;
			this.mnu1000.Text = "1000 ms";
			this.mnu1000.Click += new System.EventHandler(this.mnu1000_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 5;
			this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuAbout});
			this.menuItem4.Text = "Help";
			// 
			// mnuAbout
			// 
			this.mnuAbout.Index = 0;
			this.mnuAbout.Text = "About...";
			this.mnuAbout.Click += new System.EventHandler(this.mnuAbout_Click);
			// 
			// sb
			// 
			this.sb.Location = new System.Drawing.Point(0, 291);
			this.sb.Name = "sb";
			this.sb.Size = new System.Drawing.Size(496, 22);
			this.sb.TabIndex = 1;
			this.sb.Text = "Ready to serve";
			// 
			// tb
			// 
			this.tb.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.tb.AutoSize = false;
			this.tb.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																				  this.tbNew,
																				  this.tbOpen,
																				  this.tbSave,
																				  this.toolBarButton1,
																				  this.tbAdd,
																				  this.tbRemove,
																				  this.toolBarButton3,
																				  this.tbStates,
																				  this.tbLearn,
																				  this.tbSimulate,
																				  this.tbStop,
																				  this.toolBarButton2,
																				  this.tbWeights,
																				  this.tbAbout,
																				  this.tbQuit});
			this.tb.ButtonSize = new System.Drawing.Size(20, 20);
			this.tb.DropDownArrows = true;
			this.tb.ImageList = this.il;
			this.tb.Name = "tb";
			this.tb.ShowToolTips = true;
			this.tb.Size = new System.Drawing.Size(496, 30);
			this.tb.TabIndex = 2;
			this.tb.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tb_ButtonClick);
			// 
			// tbNew
			// 
			this.tbNew.ImageIndex = 0;
			this.tbNew.ToolTipText = "New network (Ctrl+N)";
			// 
			// tbOpen
			// 
			this.tbOpen.ImageIndex = 1;
			this.tbOpen.ToolTipText = "Open network (Ctrl+O)";
			// 
			// tbSave
			// 
			this.tbSave.ImageIndex = 2;
			this.tbSave.ToolTipText = "Save network (Ctrl+S)";
			// 
			// toolBarButton1
			// 
			this.toolBarButton1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbAdd
			// 
			this.tbAdd.ImageIndex = 8;
			this.tbAdd.ToolTipText = "Add pattern (Ins)";
			// 
			// tbRemove
			// 
			this.tbRemove.ImageIndex = 9;
			this.tbRemove.ToolTipText = "Remove pattern (Del)";
			// 
			// toolBarButton3
			// 
			this.toolBarButton3.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbStates
			// 
			this.tbStates.ImageIndex = 10;
			this.tbStates.ToolTipText = "Switch bi-polar / tri-polar network";
			// 
			// tbLearn
			// 
			this.tbLearn.ImageIndex = 5;
			this.tbLearn.ToolTipText = "Learn patterns (F5)";
			// 
			// tbSimulate
			// 
			this.tbSimulate.ImageIndex = 6;
			this.tbSimulate.ToolTipText = "Simulate network (F6)";
			// 
			// tbStop
			// 
			this.tbStop.ImageIndex = 7;
			this.tbStop.ToolTipText = "Stop (F7)";
			// 
			// toolBarButton2
			// 
			this.toolBarButton2.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbWeights
			// 
			this.tbWeights.ImageIndex = 11;
			this.tbWeights.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.tbWeights.ToolTipText = "Showing weights";
			// 
			// tbAbout
			// 
			this.tbAbout.ImageIndex = 4;
			this.tbAbout.ToolTipText = "About...";
			// 
			// tbQuit
			// 
			this.tbQuit.ImageIndex = 3;
			this.tbQuit.ToolTipText = "Quit (Ctrl+Q)";
			// 
			// il
			// 
			this.il.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.il.ImageSize = new System.Drawing.Size(20, 20);
			this.il.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("il.ImageStream")));
			this.il.TransparentColor = System.Drawing.Color.Silver;
			// 
			// sfd
			// 
			this.sfd.DefaultExt = "hopfield";
			this.sfd.Filter = "Hopfield network simulator files (*.hopfield)|*.hopfield";
			this.sfd.Title = "Save actual network to XML file";
			// 
			// ofd
			// 
			this.ofd.DefaultExt = "hopfield";
			this.ofd.Filter = "Hopfield network simulator files (*.hopfield)|*.hopfield";
			this.ofd.Title = "Open network from XML file";
			// 
			// FrmMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(496, 313);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.tb,
																		  this.sb,
																		  this.tc});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mm;
			this.MinimumSize = new System.Drawing.Size(400, 300);
			this.Name = "FrmMain";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Hopfield network simulator";
			this.tc.ResumeLayout(false);
			this.tpSet.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new FrmMain());
		}

		private void FillPatterns()
		{
			if (hnet == null) return;

			lb.Items.Clear();
			for (int i = 0; i < hnet.Patterns; i++)
			{
				if (hnet.Recognized == i)
					lb.Items.Add("[ Pattern #" + i.ToString() + " ] (recognized)");
				else
					lb.Items.Add("  Pattern #" + i.ToString());
			}
			if (hnet.Patterns > 0)
			{
			
				if (hnet.Recognized > -1)
					lb.SelectedIndex = hnet.Recognized;
				else
					lb.SelectedIndex = 0;
			}
		}

		private void DrawNet(Graphics g)
		{
			if (hnet == null) return;

			widthHalf = tpNet.Width / 2;
			height = tpNet.Height;
			gridSize = Math.Min((widthHalf - 2 * gridSpace) / hnet.DimensionX, (height - 2 * gridSpace) / hnet.DimensionY);
			width = hnet.DimensionX * gridSize + 2 * gridSpace;

			circleSize = Math.Min(tpNet.Width - width, height) / 2 - 2 * gridSpace;

			double angle = 0;
			double delta = (2 * Math.PI) / (hnet.DimensionX * hnet.DimensionY);
			diameter = (float)(2 * Math.PI * circleSize) / (hnet.DimensionX * hnet.DimensionY);
			if (diameter < 1) diameter = 1;

			circleSize -= (int)(diameter / 2);
			diameter = (float)(2 * Math.PI * circleSize) / (hnet.DimensionX * hnet.DimensionY);

			cx = width + (tpNet.Width - width) / 2 - (int)(diameter / 2);
			cy = circleSize + gridSpace;

			// neurons
			for (int y = 0; y < hnet.DimensionY; y++)
			{
				for (int x = 0; x < hnet.DimensionX; x++)
				{
					g.DrawRectangle(System.Drawing.Pens.Navy, x * gridSize + 1 + gridSpace, y * gridSize + 1 + gridSpace, gridSize, gridSize); 
					g.FillRectangle(brush[hnet.Neuron(x,y)], x * gridSize + 2 + gridSpace, y * gridSize + 2 + gridSpace, gridSize - 1, gridSize - 1); 
					if (simulated == (y * hnet.DimensionX + x))
					{
						g.FillEllipse(brushSel,
							(float)(circleSize * Math.Sin(angle) + cx),
							(float)(circleSize * Math.Cos(angle) + cy),
							diameter,
							diameter);
					}
					else
					{
						g.FillEllipse(brush[hnet.Neuron(x,y)],
							(float)(circleSize * Math.Sin(angle) + cx),
							(float)(circleSize * Math.Cos(angle) + cy),
							diameter,
							diameter);
					}
					angle += delta;
				}
			}

			// weights
			if (showWeights)
			{
				for (int k = 0; k < 16; k++)
				{
					for (int i = k; i < hnet.DimensionY * hnet.DimensionX; i+=16)
					{
						for (int j = 0; j < hnet.DimensionY * hnet.DimensionX; j++)
						{
							g.DrawLine(
								new System.Drawing.Pen(
								System.Drawing.Color.FromArgb(255-(int)(255*hnet.Weight(i,j)), (int)(255*hnet.Weight(i,j)), 0)),
								(float)(circleSize * Math.Sin(j * delta) + cx + diameter / 2),
								(float)(circleSize * Math.Cos(j * delta) + cy + diameter / 2),
								(float)(circleSize * Math.Sin(i * delta) + cx + diameter / 2),
								(float)(circleSize * Math.Cos(i * delta) + cy + diameter / 2)
								);
						}
					}
				}
			}

			// grid lines
			g.DrawLine(System.Drawing.Pens.Navy, width, 0, width, height); 
		}
		
		private void tpNet_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			DrawNet(e.Graphics);
		}

		private void DrawSet(Graphics g)
		{
			if (hnet == null) return;
			pat = lb.SelectedIndex;

			widthHalf = tpSet.Width / 2;
			height = tpSet.Height;
			gridSize = Math.Min((widthHalf - 2 * gridSpace) / hnet.DimensionX, (height - 2 * gridSpace) / hnet.DimensionY);
			width = hnet.DimensionX * gridSize + 2 * gridSpace;

			// listbox position
			lb.Width = tpSet.Width - width - 2 * gridSpace;
			lb.Left = width + gridSpace;

			// patterns
			if (hnet.Patterns > 0)
			{
				for (int y = 0; y < hnet.DimensionY; y++)
				{
					for (int x = 0; x < hnet.DimensionX; x++)
					{
						g.DrawRectangle(System.Drawing.Pens.Navy, x * gridSize + 1 + gridSpace, y * gridSize + 1 + gridSpace, gridSize, gridSize); 
						g.FillRectangle(brush[hnet.Pattern(x,y,pat)], x * gridSize + 2 + gridSpace, y * gridSize + 2 + gridSpace, gridSize - 1, gridSize - 1); 
					}
				}
			}

			// grid lines
			g.DrawLine(System.Drawing.Pens.Navy, width, 0, width, height); 
		}

		private void tpSet_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			DrawSet(e.Graphics);
		}

		private void tpNet_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if ((e.Button == MouseButtons.Left) & (mouse))
			{
				if (hnet == null) return;
				int x = (e.X - gridSpace) / gridSize;
				int y = (e.Y - gridSpace) / gridSize;
				if (((x == lx) & (y == ly)) | (x >= hnet.DimensionX)) return;
				tpNet_MouseDown(sender, e);
			}
		}

		private void tpNet_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (hnet == null) return;
			
			int x = (e.X - gridSpace) / gridSize;
			int y = (e.Y - gridSpace) / gridSize;

			// selecting neuron in grid
			if ((x >= 0) & (x < hnet.DimensionX) & (y >= 0) & (y < hnet.DimensionY))
			{
				lx = x; ly = y;
				hnet.Neuron(x, y, (hnet.Neuron(x,y) + 1));
				hnet.RecognizePattern();
				StatusRefresh();
				DrawNet(gNet);
				mouse = true;
				return;
			}
			// selecting neuron in circle
			double delta = (2 * Math.PI) / (hnet.DimensionX * hnet.DimensionY);
			for (int i = 0; i < hnet.DimensionX * hnet.DimensionY; i++)
			{
  			    x = (int) (circleSize * Math.Sin(i * delta) + cx + diameter / 2);
				y = (int) (circleSize * Math.Cos(i * delta) + cy + diameter / 2);
				if (Math.Sqrt((e.X - x) * (e.X - x) + (e.Y - y) * (e.Y - y)) <= (diameter / 2))
				{
					x = i % hnet.DimensionX;
					y = (int) i / hnet.DimensionX;
					hnet.Neuron(x, y, (hnet.Neuron(x,y) + 1));
					hnet.RecognizePattern();
					StatusRefresh();
					DrawNet(gNet);
					mouse = true;
					return;
				}
			}
		}

		private void tpSet_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if ((e.Button == MouseButtons.Left) & (mouse))
			{
 				if (hnet == null) return;
				int x = (e.X - gridSpace) / gridSize;
				int y = (e.Y - gridSpace) / gridSize;
				if (((x == lx) & (y == ly)) | (x >= hnet.DimensionX)) return;
				tpSet_MouseDown(sender, e);
			}
		}

		private void tpSet_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (hnet == null) return;

			int x = (e.X - gridSpace) / gridSize;
			int y = (e.Y - gridSpace) / gridSize;

			// selecting neuron in grid
			if ((x >= 0) & (x < hnet.DimensionX) & (y >= 0) & (y < hnet.DimensionY))
			{
				hnet.Pattern(x, y, pat, (hnet.Pattern(x,y,pat) + 1));
				hnet.RecognizePattern();
				lx = x; ly = y;
				DrawSet(gSet);
				mouse = true;
				return;
			}
		}

		private void mnuWeights_Click(object sender, System.EventArgs e)
		{
			showWeights = !showWeights;
			AllRefresh();
		}

		private void lb_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			pat = lb.SelectedIndex;
			DrawSet(gSet);
		}

		private void mnuQuit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		private void mnuNew_Click(object sender, System.EventArgs e)
		{
			FrmNet dlg = new FrmNet();
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				NetworkNew(dlg.x, dlg.y, dlg.states);
			}
		}

		private void mnuAdd_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			hnet.AddPattern();
			AllRefresh();
		}

		private void mnuRemove_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			hnet.RemovePattern(lb.SelectedIndex);
			AllRefresh();
		}

		private string GetFilenameFromPath(string path)
		{
			int p;
			p = path.LastIndexOf('\\');
			if (p >= 0) path = path.Remove(0,p+1);
			p = path.LastIndexOf('.');
			if (p >= 0) path = path.Remove(p,path.Length-p);

			return path;
		}
		
		private void mnuSave_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			sfd.FileName = fileName;
			if (sfd.ShowDialog() == DialogResult.OK)
			{
				hnet.Save(sfd.FileName);
				fileName = GetFilenameFromPath(sfd.FileName);
				tc_Click(null, null);
			}
		}

		private void mnuOpen_Click(object sender, System.EventArgs e)
		{
			if (ofd.ShowDialog() == DialogResult.OK)
			{
				sfd.FileName = ofd.FileName;
				hnet = new HNet();
				if (!hnet.Load(ofd.FileName))
				{
					fileName = "";
					MessageBox.Show("Error opening network from file '" + ofd.FileName + "'", "File open error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					InitBrushes();
					tc_Click(null, null);
				}
				else
				{
					fileName = GetFilenameFromPath(ofd.FileName);
					showWeights = (hnet.DimensionX * hnet.DimensionY <= 50);
					InitBrushes();
					tc_Click(null, null);
				}
			}
		}

		private void mnuBi_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			hnet.SetStates(2);
			AllRefresh();
		}

		private void mnuTri_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			hnet.SetStates(3);
			AllRefresh();
		}

		private void mnuLearn_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			
			sb.Text = "Learning...";
			action = true;
			MenuRefresh();
			
			string state = hnet.NeuronsToString();
			hnet.LearnReset();
			DrawNet(gNet);

			for (int i = 0; i < hnet.Patterns; i++)
			{
				hnet.Learn(i);
				DrawNet(gNet);
				Application.DoEvents();
				Thread.Sleep(3 * animDelay);
				if (!action) break;
			}

			hnet.StringToNeurons(state);
			action = false;
			AllRefresh();
		}

		private void mnuSimulate_Click(object sender, System.EventArgs e)
		{
			if (hnet == null) return;
			
			sb.Text = "Simulating...";
			action = true;
			MenuRefresh();

			//string state = hnet.NeuronsToString();
			bool proceed = true;
			bool even = true;
			int steps = 0;

			hnet.SimulateReset();
			while (proceed)
			{
				brushSel = (even=!even) ? brushSel0 : brushSel1;
				for (int i = 0; i < hnet.DimensionX * hnet.DimensionY; i++)
				{
					hnet.Simulate(i);
					simulated = i;
					steps++;
					sb.Text = "Simulating... (" + steps.ToString() + " steps)";
					DrawNet(gNet);
					Application.DoEvents();
					Thread.Sleep(animDelay);
					if (!action) 
					{
						proceed = false;
						break;
					}
					if (hnet.Stable)
					{
						proceed = false;
						break;
					}
				}
				//if ((state == hnet.NeuronsToString()) && (hnet.Stable)) proceed = false;
				//state = hnet.NeuronsToString();
			}

			simulated = -1;
			action = false;
			hnet.RecognizePattern();
			AllRefresh();
		}

		private void mnu10_Click(object sender, System.EventArgs e)
		{
			animDelay = 10;
			AllRefresh();
		}

		private void mnu100_Click(object sender, System.EventArgs e)
		{
			animDelay = 100;
			AllRefresh();
		}

		private void mnu1000_Click(object sender, System.EventArgs e)
		{
			animDelay = 1000;
			AllRefresh();
		}

		private void mnuAbout_Click(object sender, System.EventArgs e)
		{
			FrmAbout dlg = new FrmAbout();
			dlg.ShowDialog();
		}

		private void tb_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if      (e.Button == tbNew)    mnuNew_Click(null,null);
			else if (e.Button == tbOpen)   mnuOpen_Click(null,null);
			else if (e.Button == tbSave)   mnuSave_Click(null,null);
			else if (e.Button == tbAdd)    mnuAdd_Click(null,null);
			else if (e.Button == tbRemove) mnuRemove_Click(null,null);
			else if (e.Button == tbWeights)mnuWeights_Click(null,null);
			else if (e.Button == tbLearn)  mnuLearn_Click(null,null);
			else if (e.Button == tbSimulate)mnuSimulate_Click(null,null);
			else if (e.Button == tbAbout)  mnuAbout_Click(null,null);
			else if (e.Button == tbQuit)   mnuQuit_Click(null,null);
			else if (e.Button == tbStop)   mnuStop_Click(null,null);
			else if (e.Button == tbStates)
			{
				if (hnet == null) return;
				hnet.SetStates( (hnet.States == 2) ? 3 : 2 );
				AllRefresh();
			}
		}

		private void mnuStop_Click(object sender, System.EventArgs e)
		{
			action = false;
		}

		private void tc_SizeChanged(object sender, System.EventArgs e)
		{
			gNet = tpNet.CreateGraphics();
			gSet = tpSet.CreateGraphics();
		}

		private void tc_Click(object sender, System.EventArgs e)
		{
			tc_SizeChanged(null,null);
			AllRefresh();
		}

		private void mnuRandom_Click(object sender, System.EventArgs e)
		{
			string state = hnet.NeuronsToString();
			FrmTest dlg = new FrmTest(true, hnet);
			dlg.ShowDialog();
			hnet.StringToNeurons(state);
			AllRefresh();
		}

		private void mnuWhole_Click(object sender, System.EventArgs e)
		{
			string state = hnet.NeuronsToString();
			FrmTest dlg = new FrmTest(false, hnet);
			dlg.ShowDialog();
			hnet.StringToNeurons(state);
			AllRefresh();
		}

		private void tpNet_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			mouse = false;
		}
	}
}
