using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;

namespace Hopfield3
{
	/// <summary>
	/// Summary description for FrmTest.
	/// </summary>
	public class FrmTest : System.Windows.Forms.Form
	{
		private System.Windows.Forms.ListBox lb;
		private System.Windows.Forms.Label lTest;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label lNumber;
		private System.Windows.Forms.Label lRecognized;
		private System.Windows.Forms.Label lAverage;
		private System.Windows.Forms.Label lCycling;
		private System.Windows.Forms.ProgressBar pb;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private bool isRandom;
		private System.Timers.Timer timer;
		private HNet tnet;

		private System.Windows.Forms.Button bStart;
		private System.Windows.Forms.Button bExit;

		private ulong[] recognized;
		private ulong total;
		private ulong tests;
		private ulong totalIters;
		private ulong totalRecognized;
		private ulong totalCycling;
		private bool testing;

		private Random rand;

		public FrmTest(bool random, HNet net)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.isRandom = random;
			this.tnet = net;

			if (isRandom) 
			{
				this.Text = "Testing random states";
			}
			else
			{
				this.Text = "Testing all states";
			}

			rand = new Random();
			recognized = new ulong[tnet.Patterns];
			total = (ulong)Math.Pow(tnet.States, tnet.DimensionX * tnet.DimensionY);

			lTest.Text = "Testing space size = " + tnet.States.ToString() + "^" + ((int)tnet.DimensionX * tnet.DimensionY).ToString();
			
			if (total == 0)
			{
				lTest.Text += " >> 1.8E19";
				if (!isRandom)
				{
					MessageBox.Show("Testing space is out of supported size, only random testing is possible!", "Testing space too big", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					pb.Visible = false;
					bStart.Enabled = false;
				}
			}
			else
				lTest.Text += " = " + total.ToString();

			timer_Elapsed(null,null);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FrmTest));
			this.lb = new System.Windows.Forms.ListBox();
			this.lTest = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.lNumber = new System.Windows.Forms.Label();
			this.lRecognized = new System.Windows.Forms.Label();
			this.lAverage = new System.Windows.Forms.Label();
			this.lCycling = new System.Windows.Forms.Label();
			this.pb = new System.Windows.Forms.ProgressBar();
			this.timer = new System.Timers.Timer();
			this.bStart = new System.Windows.Forms.Button();
			this.bExit = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.timer)).BeginInit();
			this.SuspendLayout();
			// 
			// lb
			// 
			this.lb.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.lb.IntegralHeight = false;
			this.lb.Location = new System.Drawing.Point(5, 176);
			this.lb.MultiColumn = true;
			this.lb.Name = "lb";
			this.lb.Size = new System.Drawing.Size(280, 168);
			this.lb.TabIndex = 0;
			// 
			// lTest
			// 
			this.lTest.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.lTest.BackColor = System.Drawing.SystemColors.ControlDark;
			this.lTest.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.lTest.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lTest.ForeColor = System.Drawing.SystemColors.Window;
			this.lTest.Location = new System.Drawing.Point(8, 8);
			this.lTest.Name = "lTest";
			this.lTest.Size = new System.Drawing.Size(276, 17);
			this.lTest.TabIndex = 1;
			this.lTest.Text = "Testing space size...";
			this.lTest.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.label2.Location = new System.Drawing.Point(8, 40);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(48, 16);
			this.label2.TabIndex = 2;
			this.label2.Text = "Test #";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.label3.Location = new System.Drawing.Point(8, 136);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(272, 16);
			this.label3.TabIndex = 3;
			this.label3.Text = "Possibly cycling states (more than 1000 iterations)";
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.label4.Location = new System.Drawing.Point(8, 72);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(128, 16);
			this.label4.TabIndex = 4;
			this.label4.Text = "Recognized patterns";
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.label5.Location = new System.Drawing.Point(8, 104);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(248, 16);
			this.label5.TabIndex = 5;
			this.label5.Text = "Average number of iterations";
			// 
			// lNumber
			// 
			this.lNumber.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lNumber.Location = new System.Drawing.Point(8, 56);
			this.lNumber.Name = "lNumber";
			this.lNumber.Size = new System.Drawing.Size(128, 16);
			this.lNumber.TabIndex = 6;
			this.lNumber.Text = "0";
			// 
			// lRecognized
			// 
			this.lRecognized.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lRecognized.Location = new System.Drawing.Point(8, 88);
			this.lRecognized.Name = "lRecognized";
			this.lRecognized.Size = new System.Drawing.Size(120, 16);
			this.lRecognized.TabIndex = 7;
			this.lRecognized.Text = "0%";
			// 
			// lAverage
			// 
			this.lAverage.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lAverage.Location = new System.Drawing.Point(8, 120);
			this.lAverage.Name = "lAverage";
			this.lAverage.Size = new System.Drawing.Size(248, 16);
			this.lAverage.TabIndex = 8;
			this.lAverage.Text = "0";
			// 
			// lCycling
			// 
			this.lCycling.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(238)));
			this.lCycling.ForeColor = System.Drawing.Color.Red;
			this.lCycling.Location = new System.Drawing.Point(8, 152);
			this.lCycling.Name = "lCycling";
			this.lCycling.Size = new System.Drawing.Size(248, 16);
			this.lCycling.TabIndex = 9;
			this.lCycling.Text = "0";
			// 
			// pb
			// 
			this.pb.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.pb.Location = new System.Drawing.Point(8, 26);
			this.pb.Name = "pb";
			this.pb.Size = new System.Drawing.Size(276, 8);
			this.pb.TabIndex = 10;
			// 
			// timer
			// 
			this.timer.Enabled = true;
			this.timer.Interval = 200;
			this.timer.SynchronizingObject = this;
			this.timer.Elapsed += new System.Timers.ElapsedEventHandler(this.timer_Elapsed);
			// 
			// bStart
			// 
			this.bStart.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
			this.bStart.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.bStart.Location = new System.Drawing.Point(208, 40);
			this.bStart.Name = "bStart";
			this.bStart.TabIndex = 11;
			this.bStart.Text = "&Start";
			this.bStart.Click += new System.EventHandler(this.bStart_Click);
			// 
			// bExit
			// 
			this.bExit.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
			this.bExit.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.bExit.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.bExit.Location = new System.Drawing.Point(208, 67);
			this.bExit.Name = "bExit";
			this.bExit.TabIndex = 12;
			this.bExit.Text = "&Exit";
			this.bExit.Click += new System.EventHandler(this.bExit_Click);
			// 
			// FrmTest
			// 
			this.AcceptButton = this.bStart;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.bExit;
			this.ClientSize = new System.Drawing.Size(292, 349);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.bExit,
																		  this.bStart,
																		  this.pb,
																		  this.lCycling,
																		  this.lAverage,
																		  this.lRecognized,
																		  this.lNumber,
																		  this.label5,
																		  this.label4,
																		  this.label3,
																		  this.label2,
																		  this.lTest,
																		  this.lb});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MinimumSize = new System.Drawing.Size(300, 300);
			this.Name = "FrmTest";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Testing";
			((System.ComponentModel.ISupportInitialize)(this.timer)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			lNumber.Text = tests.ToString();		
			if (tests > 0)
			{
				lRecognized.Text = ((int)((100 * totalRecognized) / tests)).ToString() + "%";
				lAverage.Text = ((int)(totalIters / tests)).ToString();
				if (isRandom)
					pb.Value = rand.Next(100); else
					pb.Value = (int)((100 * tests) / total);
				lb.Items.Clear();
				for (int i = 0; i < tnet.Patterns; i++)
				{
					lb.Items.Add(
						" Pattern #" + i.ToString() +
						" \t= " + recognized[i].ToString() +
						" \t(" + (Math.Round((double)100*recognized[i]/tests)).ToString() + "%)");
				}
			}
			lCycling.Text = totalCycling.ToString();
		}

		private void bStart_Click(object sender, System.EventArgs e)
		{
			int i;

			tnet.LearnReset();
			for (i = 0; i < tnet.Patterns; i++) tnet.Learn(i);
			
			const int max_iters = 1000;
			
			testing = true;
			bStart.Enabled = false;
			tests = 0;

			//string state;
			string prevState;
			bool proceed;
			int steps;
			Random rnd = new Random();

			tnet.ResetState();

			while(testing)
			{
				if (isRandom) tnet.RandomState(rnd);

				steps = 0;
				proceed = true;
				prevState = tnet.NeuronsToString();
				//lb.Items.Add(prevState);

				tnet.SimulateReset();
				while ((proceed) & (steps < max_iters))
				{
					for (i = 0; i < tnet.DimensionX * tnet.DimensionY; i++)
					{
						tnet.Simulate(i);
						steps++;
						if (tnet.Stable)
						{
							proceed = false;
							break;
						}
					}
					//if (state == tnet.NeuronsToString()) proceed = false;
					//state = tnet.NeuronsToString();
				}

				tnet.RecognizePattern();
				tests++;

				totalIters += (ulong)steps;
				if (tnet.Recognized > -1)
				{
					totalRecognized++;
					recognized[tnet.Recognized]++;
				}

				if (steps == max_iters)
				{
					totalCycling++;
				}

				Application.DoEvents();

				if (!isRandom)
				{
					if (tests >= total)
						testing = false;
					else
					{
						tnet.StringToNeurons(prevState);
						tnet.NextState();	
					}
				} 
			}

			pb.Visible = false;
		}

		private void bExit_Click(object sender, System.EventArgs e)
		{
			testing = false;
		}
	}
}
