using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Xml;

namespace Hopfield3
{
	public class HNet : System.ComponentModel.Component
	{
		public const int max_states = 8;
		
		private System.ComponentModel.Container components = null;
		private int[] neurons;
		private ArrayList patterns;
		private double[,] weights;
		private int[,,,] ratios;
		private int dimensionX;
		private int dimensionY;
		private int states;
		private int recognized = -1;
		private int stable;

		public const int invalidValue = 0;

		public int DimensionX
		{
			get { return dimensionX; }
		}

		public int DimensionY
		{
			get { return dimensionY; }
		}

		public int States
		{
			get { return states; }
		}

		public bool Stable
		{
			get { return (stable >= dimensionX * dimensionY); }
		}

		public void SetStates(int states)
		{
			if (this.states == states) return;
			if (this.states < states)
			{
				this.states = states;
			}
			else
			{
				this.states = states;
				for (int i = 0; i < dimensionX * dimensionY; i++)
					neurons[i] = neurons[i] % states;
				for (int i = 0; i < patterns.Count; i++)
				{
					for (int j = 0; j < dimensionX * dimensionY; j++)
					{
						((int[])patterns[i])[j] = ((int[])patterns[i])[j] %	states;
		}}}}

		public int Patterns
		{
			get { return patterns.Count; }
		}

		public int Recognized
		{
			get { return recognized; }
		}

		public void LearnReset()
		{
			for (int i = 0; i < dimensionX * dimensionY; i++)			
				for (int j = 0; j < dimensionX * dimensionY; j++)			
				{
					weights[i,j] = 0;
					for (int k = 0; k < max_states; k++)
						for (int l = 0; l < max_states; l++)
						{
							ratios[i,j,k,l] = 0;
						}
				}
		}

		public void Learn(int p)
		{
			for (int i = 0; i < dimensionX * dimensionY; i++)			
				neurons[i] = ((int[])patterns[p])[i];
			
			for (int i = 0; i < dimensionX * dimensionY; i++)			
				for (int j = 0; j < dimensionX * dimensionY; j++)			
				{
					if (i != j)
					{
						ratios[i,j,neurons[i],neurons[j]]++;
						weights[i,j] = (neurons[i] == neurons[j]) ? 1 : 0;
					}
				}
		}

		public void SimulateReset()
		{
			stable = 0;
		}

		public void Simulate(int n)
		{
			// DEBUG - Wij matrix 
			/*
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				for (int j = 0; j < dimensionX * dimensionY; j++)
				{
					string temp = "W" + i.ToString() + j.ToString() + " = ";
					for (int k = 0; k < max_states; k++)
					{
						for (int l = 0; l < max_states; l++)
						{
							temp += ratios[i,j,k,l].ToString() + " ";
						}
						temp += " |  ";
					}
					temp += ".";
				}
			}
			*/

			int[] sts = new int[states];
			for (int i = 0; i < dimensionX * dimensionY; i++)			
			{
				if (i != n)
				{
					for (int j = 0; j < states; j++)
					{
						sts[j] += ratios[n,i,j,neurons[i]];
					}
				}
			}
			int max = 0;
			int index = neurons[n];

			//string temp = "";

			for (int i = 0; i < states; i++)
			{
				// DEBUG - sums over n-element vector
				//temp += sts[i].ToString() + " ";
				if (max == sts[i])
				{
					if (i == neurons[n])
						index = neurons[n];
					else
						index = i;
				}
				if (max < sts[i])
				{
					max = sts[i];
					index = i;
				}
			}
			if (neurons[n] == index) 
				stable++;
			else
			{
				stable = 0;
				neurons[n] = index;
			}
		}

		public void RecognizePattern()
		{
			recognized = -1;
			bool same;
			for (int p = 0; p < patterns.Count; p++)
			{
				same = true;
				for (int i = 0; i < dimensionX * dimensionY; i++)			
				{
					if (neurons[i] != ((int[])patterns[p])[i])
					{
						same = false;
						break;
					}
				}
				if (same) recognized = p;
			}
		}
		
		public int Neuron(int x, int y)
		{
			if ((x >= 0) & (x < dimensionX) & (y >= 0) & (y < dimensionY)) 
				return neurons[y * dimensionX + x];
			else
				return invalidValue;
		}

		public void Neuron(int x, int y, int val)
		{
			if ((x >= 0) & (x < dimensionX) & (y >= 0) & (y < dimensionY)) 
				neurons[y * dimensionX + x] = val % states;
		}

		public int Pattern(int x, int y, int p)
		{
			if ((x >= 0) & (x < dimensionX) & (y >= 0) & (y < dimensionY) & (p >= 0) & (p < patterns.Count)) 
				return ((int[])patterns[p])[y * dimensionX + x];
			else
				return invalidValue;
		}

		public void Pattern(int x, int y, int p, int val)
		{
			if ((x >= 0) & (x < dimensionX) & (y >= 0) & (y < dimensionY) & (p >= 0) & (p < patterns.Count)) 
				((int[])patterns[p])[y * dimensionX + x] = val % states;
		}

		public double Weight(int n1, int n2)
		{
			if ((n1 >= 0) & (n1 < dimensionX * dimensionY) &
				(n2 >= 0) & (n2 < dimensionX * dimensionY)) 
				return weights[n1,n2];
			else
				return invalidValue;
		}

		public HNet(System.ComponentModel.IContainer container)
		{
			container.Add(this);
			InitializeComponent();
			InitNetwork(2,2,2);
		}

		public HNet()
		{
			InitializeComponent();
			InitNetwork(2,2,2);
		}

		public HNet(int dimensionX, int dimensionY, int states)
		{
			InitializeComponent();
			InitNetwork(dimensionX, dimensionY, states);
		}

		public void InitNetwork(int dimensionX, int dimensionY, int states)
		{
			this.dimensionX = dimensionX;
			this.dimensionY = dimensionY;
			this.states = states;
			neurons = new int[dimensionX * dimensionY];
			patterns = new ArrayList();
			weights = new double[dimensionX * dimensionY, dimensionX * dimensionY];
			ratios = new int[dimensionX * dimensionY, dimensionX * dimensionY,max_states,max_states];
		}

		public void AddPattern()
		{
			int[] pattern;
			pattern = new int[dimensionX * dimensionY];
			patterns.Add(pattern);
			RecognizePattern();
		}

		public void RemovePattern(int p)
		{
			if ((p < 0) | (p >= patterns.Count)) return;
			patterns.RemoveAt(p);
			RecognizePattern();
		}

		public void RandomizeNetwork()
		{
			Random rnd = new Random();
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				neurons[i] = rnd.Next(states);
			}
			for (int i = 0; i < dimensionX * dimensionY; i++)
			for (int j = 0; j < dimensionX * dimensionY; j++)
			{
				weights[i,j] = rnd.NextDouble();
			}
		}

		public void ResetState()
		{
			for (int i = 0; i < dimensionX * dimensionY; i++) neurons[i] = 0;
		}

		public void NextState()
		{
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				if (neurons[i] < states - 1)
				{
					neurons[i]++;
					return;
				}
				else
				{
					neurons[i] = 0;
				}
			}
		}

		public void RandomState(Random rnd)
		{
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				neurons[i] = rnd.Next(states);
			}
		}

		private string PatternToString(int p)
		{
			string s = "";
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				s += ((int[])patterns[p])[i].ToString();
			}
			return s;
		}

		public string NeuronsToString()
		{
			string s = "";
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				s += (neurons)[i].ToString();
			}
			return s;
		}

		private void StringToPattern(string s)
		{
			if (s.Length < dimensionX * dimensionY) return;
			int[] pattern;
			pattern = new int[dimensionX * dimensionY];
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				try
				{
					pattern[i] = System.Convert.ToInt32(s[i]) % states;	
				}
				catch {}
			}
			patterns.Add(pattern);
		}
		
		public void StringToNeurons(string s)
		{
			if (s.Length < dimensionX * dimensionY) return;
			for (int i = 0; i < dimensionX * dimensionY; i++)
			{
				try
				{
					neurons[i] = System.Convert.ToInt32(s[i]) % states;	
				}
				catch {}
			}
		}

		public void Save(string filename)
		{
			XmlTextWriter xw = new XmlTextWriter(filename, System.Text.Encoding.ASCII);

			xw.WriteStartDocument();
			xw.WriteComment("Hopfield network simulator XML file");

			xw.WriteStartElement("Network");
				xw.WriteStartElement("Setup");
					xw.WriteStartElement("DimensionX");
					xw.WriteString(dimensionX.ToString());
					xw.WriteEndElement();
					xw.WriteStartElement("DimensionY");
					xw.WriteString(dimensionY.ToString());
					xw.WriteEndElement();
					xw.WriteStartElement("States");
					xw.WriteString(states.ToString());
					xw.WriteEndElement();
					xw.WriteStartElement("Neurons");
					xw.WriteString(NeuronsToString());
					xw.WriteEndElement();
				xw.WriteEndElement();

				xw.WriteStartElement("Patterns");
				for (int i = 0; i < patterns.Count; i++)
				{
					xw.WriteStartElement("Data");
					xw.WriteString(PatternToString(i));
					xw.WriteEndElement();
				}
				xw.WriteEndElement();
			xw.WriteEndElement();

			xw.Close();
		}

		public bool Load(string filename)
		{
			try
			{
				XmlTextReader xr = new XmlTextReader(filename);

				int dx = 0;
				int dy = 0;
				int st = 0;
				string ns = "";
				ArrayList list = new ArrayList();

				xr.MoveToContent();
				string s;
				while (!xr.EOF)
				{
					s = xr.ReadString();
					if (xr.Name == "DimensionX")	dx = System.Convert.ToInt32(s);
					if (xr.Name == "DimensionY")	dy = System.Convert.ToInt32(s);
					if (xr.Name == "States")		st = System.Convert.ToInt32(s);
					if (xr.Name == "Neurons")		ns = s;
					if (xr.Name == "Data")			list.Add(s);
					xr.Read();
				}

				xr.Close();
				
				InitNetwork(dx, dy, st);
				for (int i = 0; i < list.Count; i++)
					StringToPattern((string)list[i]);

				LearnReset();
				for (int i = 0; i < patterns.Count; i++)
					Learn(i);

				StringToNeurons(ns);
				RecognizePattern();

				return true;
			}
			catch
			{
				return false;
			}
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}
